###########################################################
## This is an R file that forms part of the research
## project "An Exponential Chi-Squared QMLE for Log-GARCH
## Models Via the ARMA Representation" by Christian Francq
## and Genaro Sucarrat
##
## First created: 20 June 2013, Madrid
## Last changed: 21 August 2017, Oslo
##
## Contents:
##
## 1 INITIATE
## 2 AUXILIARY FUNCTIONS
## 3 ANALYSIS OF EURO-AREA INFLATION
##
###########################################################


###########################################################
##1 INITIATE
###########################################################

##set working directory (i.e. where the data is located):
setwd(choose.dir())

##load necessary packages:
library(lgarch)
library(gets)
library(tseries)


###########################################################
##2 AUXILIARY FUNCTIONS
###########################################################

##================================================
#Perform k Ljung-Box tests:
gBox.test <- function(x, lag=1:35, type=c("Ljung-Box", "Box-Pierce"),
fitdf=0)
{
type <- match.arg(type)
out <- matrix(NA, length(lag), 2)
xadj <- as.vector(na.omit(x))
t <- 0
for(i in lag){
  tmp <- Box.test(xadj, lag=i, type=type, fitdf=fitdf)
  t <- t + 1
  out[t,1] <- tmp$stat
  out[t,2] <- tmp$p.value
}
autocorrs <- acf(xadj, lag.max=max(lag), plot=FALSE, na.action=na.omit)
out <- cbind(coredata(autocorrs$acf[2:I(max(lag)+1)])[lag], out)
colnames(out) <- c("AutoCorr", "Q-stat", "p-val")
rownames(out) <- lag
return(out)
} ## end gBox.test

##================================================
##rounding as taught in many elementary schools:
ground <- function(x, digits=0, eps=.Machine$double.eps)
{
  if( all(!is.na(x)) ){
    x[x > 0] <- x[x > 0] + eps
    x[x < 0] <- x[x < 0] + eps
    x <- round(x, digits=digits)
  }else{
    x <- NA
  }
  return(x)
} #end ground

##================================================
##LaTeX print of log-garch model:
latex <- function(object, digits=3, ...)
{
  coefs <- coef(object)
  coefsNames <- names(coefs)
  stderrs <- sqrt(diag(vcov(object)))
  
  txt <- paste(ifelse(coefs[1] < 0, " - ", " "),
          format(ground(abs(coefs[1]), digits=digits),
          nsmall=digits), sep="")

  for(i in 2:length(coefs) ){
    txt <- paste(txt,
      ifelse(coefs[i] < 0, " - ", " + "), "\\underset{(",
      format(ground(stderrs[i], digits=digits), nsmall=digits), ")",
      ifelse(abs(coefs[i])/stderrs[i] > 2, "^*", ""), "}{",
      format(ground(abs(coefs[i]), digits=digits), nsmall=digits), "}",
      coefsNames[i], sep="")
  }

  txt <- paste(txt, "\n", sep=" ")
  cat(txt)

}#end latex


###########################################################
##3 ANALYSIS OF EURO-AREA INFLATION
###########################################################

##import data, prepare analysis:
mhicp <- read.csv("hicp-euro-area.csv") #source: European Central Bank (http://www.ecb.int/)
hicp.n <- nrow(mhicp)
hicp.date <- seq(as.Date("2001-01-01"), as.Date("2013-06-1"),
  by = "month")
hicp <- zoo(as.numeric(mhicp[,2])[hicp.n:1], order.by=hicp.date)
hicp.infl <- (coredata(diff(hicp))/coredata(hicp[-hicp.n])) * 100
hicp.infl <- zoo(hicp.infl, order.by=hicp.date[-1])
hicp.infl12 <- (coredata(diff(hicp, lag=12))/coredata(hicp)[-I((hicp.n-11):hicp.n)]) * 100
hicp.infl12 <- zoo(hicp.infl12, order.by=hicp.date[-I(1:12)])

##ar(12) model:
hicp.inflMod01 <- arx(hicp.infl12, mc=1, ar=1:12,
  vcov.type="white")
gBox.test(hicp.inflMod01$resids)
gBox.test(hicp.inflMod01$resids^2)

##prepare estimation of log-garch model:
hicp12.resids01 <- hicp.inflMod01$resids
mLoge2 <- cbind(log(hicp.inflMod01$resids^2) - mean(log(hicp.inflMod01$resids^2)))
for(i in 1:12){
  mLoge2 <- cbind(mLoge2, glag(mLoge2[,1], k=i))
}
mLoge2 <- mLoge2[,-1]
tmp.names <- paste("loge2lag", 1:12, sep="")
colnames(mLoge2) <- tmp.names

##lgarch(12,1):
hicp.VarMod01 <- lgarch(hicp12.resids01[-c(1:12)],
  xreg=mLoge2[-c(1:12),-c(1)], method="cex2")
gBox.test(residuals(hicp.VarMod01))
gBox.test(residuals(hicp.VarMod01)^2)
hicp.VarMod01logLik <- as.numeric(logLik(hicp.VarMod01))
latex(hicp.VarMod01)

##simplified log-garch:
hicp.VarMod02 <- lgarch(hicp12.resids01[-c(1:12)],
  xreg=mLoge2[-c(1:12),-c(1,2,3,4,7,8,9,10,12)], method="cex2")
gBox.test(residuals(hicp.VarMod02))
gBox.test(residuals(hicp.VarMod02)^2)
hicp.VarMod02logLik <- as.numeric(logLik(hicp.VarMod02))
coef(hicp.VarMod02)/sqrt(diag(vcov(hicp.VarMod02)))
latex(hicp.VarMod02)

##ordinary garch models:
hicp.VarMod03 <- garch(coredata(hicp12.resids01[-c(1:12)]),
  order=c(1,12), trace=FALSE)
hicp.VarMod03
gBox.test(residuals(hicp.VarMod03))
gBox.test(residuals(hicp.VarMod03)^2)
hicp.VarMod03logLik <- as.numeric(logLik(hicp.VarMod03))

hicp.VarMod04 <- garch(coredata(hicp12.resids01[-c(1:12)]),
  order=c(2,2), trace=FALSE)
hicp.VarMod04
gBox.test(residuals(hicp.VarMod04))
gBox.test(residuals(hicp.VarMod04)^2)
hicp.VarMod04logLik <- as.numeric(logLik(hicp.VarMod04))
sqrt(diag(vcov(hicp.VarMod04)))

hicp.VarMod05 <- garch(coredata(hicp12.resids01[-c(1:12)]),
  order=c(1,1), trace=FALSE)
hicp.VarMod05
gBox.test(residuals(hicp.VarMod05))
gBox.test(residuals(hicp.VarMod05)^2)
hicp.VarMod04logLik <- as.numeric(logLik(hicp.VarMod05))
latex(hicp.VarMod05)
